import java.awt.*;
import java.awt.geom.*;

/**
 * This class represents a ball that bounces around a BallCourt.
 * 
 * @author Joel Ross & YOUR_NAME_HERE
 */
public class Ball
{
    private int diameter; //the diameter of the ball
    private double xPosition; //the ball's xPosition
    private double yPosition; //yte ball's yPosition
    private double xSpeed; //the speed the ball is moving along the x-axis
    private double ySpeed; //the speed the ball is moving along the y-axis
    private Color color; //the color of the ball
    private int colorIndex; //the index of the color of the ball in the COLORS array

    //More constants are defined at the bottom of the class!
    
    /**
     * Constructor for objects of class BouncingBall
     */
    public Ball()
    {
        diameter = 40;
        xPosition = LEFT_BOUND + Math.random()*(BallCourt.COURT_WIDTH-diameter);
        yPosition = TOP_BOUND + Math.random()*(BallCourt.COURT_HEIGHT-diameter);
        xSpeed = Math.random()*7;
        ySpeed = Math.random()*7;

        //TODO - Make this variable more random so not all the balls are the same color!
        colorIndex = 0;
        color = COLORS[0];
    }

    /**
     * Moves the ball its speed; the ball bounces off of the bounds of the BallCourt, 
     * and changes color when it does so!
     */
    public void move()
    {
        //move our speed
        xPosition += xSpeed;
        yPosition += ySpeed;
        
        //Check to see if we hit a wall
        //TODO -- CHANGE TO A NEW COLOR WHEN WE HIT A WALL!
        if(xPosition < LEFT_BOUND) //if we're on the edge
        {
            xPosition = LEFT_BOUND; //put back at the edge
            xSpeed = -xSpeed; //turn around
        }
        else if(xPosition+diameter > RIGHT_BOUND) //if we're on the edge
        {
            xPosition = RIGHT_BOUND-diameter; //put back at the edge
            xSpeed = -xSpeed; //turn around
        }
        if(yPosition < TOP_BOUND) //if we're on the edge
        {
            yPosition = TOP_BOUND; //put back at the edge
            ySpeed = -ySpeed; //turn around
        }
        else if(yPosition+diameter > BOTTOM_BOUND)
        {
            yPosition = BOTTOM_BOUND-diameter;
            ySpeed = -ySpeed; //turn around
        }
    }
    
    /**
     * Gets the current x position of the ball
     * @return the x position of the ball
     */
    public int getX()
    {
        return (int)xPosition;
    }

    /**
     * Gets the current y position of the ball
     * @return the y position of the ball
     */
    public int getY()
    {
        return (int)yPosition;
    }
    
    /**
     * Gets the diamater of the ball
     * @return the diameter of the ball
     */
    public int getDiameter()
    {
        return diameter;
    }
    
    /**
     * Gets the current color of the ball
     * @return the color of the ball as a Color object
     */
    public Color getColor()
    {
        return color;
    }
    
    /**
     * Determines whether this ball is touching (overlaps) with the given other ball.
     * @param other the other ball to check again
     * @return whether or not the two balls are touching
     */
    public boolean isTouching(Ball other)
    {
        double distance = Math.sqrt(
            (this.xPosition-other.getX())*(this.xPosition-other.getX()) +
            (this.yPosition-other.getY())*(this.yPosition-other.getY())); //pythagorean theorem
            
        return distance < (this.diameter+other.getDiameter()); //touching if distance is less than sum of the diameters
    }
    
    /**
     * Adjusts our speed as we bounce off another ball
     * @param other the ball we have bounced off
     */
    public void bounceOff(Ball other)
    {
        //reverse direction
        xSpeed = -xSpeed;
        ySpeed = -ySpeed;

        double distance = Math.sqrt(
            (this.xPosition-other.getX())*(this.xPosition-other.getX()) +
            (this.yPosition-other.getY())*(this.yPosition-other.getY())); //pythagorean theorem

        if(distance < this.diameter+other.getDiameter()) //if we are still touching
        {
            //move us something approximating the distance away. This should mostly put us out of touch
            xPosition += (Math.abs(xSpeed)/xSpeed)*distance/2*Math.sqrt(2); 
            yPosition += (Math.abs(ySpeed)/ySpeed)*distance/2*Math.sqrt(2);
        }
    }
    
    /****CONSTANTS FOR YOUR CONVENIENCE --- DO NOT MODIFY****/
    //a list of less garish colors to use when drawing the balls. Generated by Chuck Hommel
    //This is short-hand for creating an array of Color objects (a Color[])
    public static final Color[] COLORS = {
            new Color(255,38,38),
            new Color(255,168,38),
            new Color(212,255,38),
            new Color(82,255,38),
            new Color(38,255,125),
            new Color(38,255,255),
            new Color(38,125,255),
            new Color(82,38,255),
            new Color(212,38,255),
            new Color(255,38,168),
        }; 

    //convenience constants for court boundaries--these make life a bit easier
    private final int TOP_BOUND = BallCourt.Y_CORNER;
    private final int LEFT_BOUND = BallCourt.X_CORNER;
    private final int BOTTOM_BOUND = BallCourt.Y_CORNER + BallCourt.COURT_HEIGHT;
    private final int RIGHT_BOUND = BallCourt.X_CORNER + BallCourt.COURT_WIDTH;
    /**** END CONSTANTS ****/

}
