import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * A frame to hold and display a BallCourt
 * 
 * @author Joel Ross
 */
public class BallCourtFrame implements ActionListener
{
    private JFrame frame; //the actual frame we'll be showing
    private BallCourt court; //the ballcourt we'll be drawing
    private Timer animator; //a timer to control animation
    
    /**
     * Creates a new BallCourtFrame object.
     */
    public BallCourtFrame()
    {
        frame = new JFrame("Bouncing Balls"); //make the JFrame 
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		court = new BallCourt();
		court.setPreferredSize(new Dimension(2*BallCourt.X_CORNER+BallCourt.COURT_WIDTH, 2*BallCourt.Y_CORNER+BallCourt.COURT_HEIGHT));
		frame.getContentPane().add(court); //put the court in the frame
		
		animator = new Timer(30,this); //create the timer, with this object controlling it

		frame.pack(); //make everything the preferred size
		frame.setVisible(true); //show the frame
		
		startAnimation(); //start the animation!

    }
    
    /**
     * Method that controls the animation timer
     */
    public void actionPerformed(ActionEvent e)
    {
        court.repaint();
    }
    
    /**
     * Starts the animation timer
     */
    public void startAnimation()
    {
        animator.start();
    }

    /**
     * Stops the animation timer
     */
    public void pauseAnimation()
    {
        animator.stop();
    }
    
}
